function nma_HW5_prob2_3D_animation
%
% 3D animation of solution to wave PDE on circular diaphram
% by Nasser Abbasi. Hw5, problem 2. MAE200B. UCI
%
clear all; close all;

a = 2;
b = 2;
c = 4;
nPoints = 50;
tfinal  = 5;    % how long to run animation
dt      = 0.01; %make this smaller for better simulation but slower

[THETA,R]=meshgrid(linspace(0,2*pi,nPoints),linspace(-a,a,nPoints));
[X,Y]=pol2cart(THETA,R);

xPoints = size(X,2);
yPoints = size(X,1);

for t = 0:dt:tfinal
    U = zeros(nPoints);
    
    m = 0;  %there is only solution for m=0
    for n = 1:5
        Zmn = besselzero(m,n,1);
        Zmn = Zmn(n);
        Bnm = B_nm(m,n,a,Zmn);
        U   = U + Bnm*cos(2*THETA).*cos(Zmn/a *c*t) .* ...
                  besselj(m,Zmn/a .*R);
    end
    
    clf;
    surf(X,Y,real(U));   colormap(jet);
    axis([-Inf Inf -Inf Inf -1 1]);
    if t==0
        title('position at t=0')
    else
        title(sprintf('time=%1.3f sec',t))
    end
    xlabel('x'); ylabel('y'); zlabel('U(r,theta)');
    view(20,25);
    drawnow;
    %if t==0
    %   figure;
    %end
end

%%%%%%%%
%
%%%%%%%%
function Bnm=B_nm(m,n,a,Zmn)

Bnm = quad(@(r) r .* (1- (2.*r/a -1).^2 ) .* besselj(m,Zmn/a .*r),0, a);
Bnm=Bnm/( pi* a^2/2 * (besselj(m+1,Zmn))^2 );

%%%%%%%
%
%%%%%%%
function Cnm=C_nm(m,n,a,Zmn)

Bnm = quad(@(r) r .* (-16 * r .* (-r+a+cos(pi*m).^2.*r - cos(pi*m).^2*a))/(a^2*(m^2-4) ) .* besselj(m,Zmn/a .*r),0, a);

Cnm = quad(@(r)...
    r .*(16*r.^2*sin(m*pi)*cos(m*pi)/(a^2*(m^2-4)) .* besselj(m,Zmn/a .*r)),0, a);
Cnm=Cnm/( pi* a^2/2 * (besselj(m+1,Zmn))^2 )


% This function below I borrowed from the net to find the
% zeros of bessel functions.
function x=besselzero(n,k,kind)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% besselzero.m
%
% Find first k positive zeros of the Bessel function J(n,x) or Y(n,x)
% using Halley's method.
%
% Written by: Greg von Winckel - 01/25/05
% Contact: gregvw(at)chtm(dot)unm(dot)edu
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

k3=3*k;

x=zeros(k3,1);

for j=1:k3

    % Initial guess of zeros
    x0=1+sqrt(2)+(j-1)*pi+n+n^0.4;

    % Do Halley's method
    x(j)=findzero(n,x0,kind);

    if x(j)==inf
        error('Bad guess.');
    end

end

x=sort(x);
dx=[1;abs(diff(x))];
x=x(dx>1e-8);

x=x(1:k);

function x=findzero(n,x0,kind)

n1=n+1;     n2=n*n;

% Tolerance
tol=1e-12;

% Maximum number of times to iterate
MAXIT=100;

% Initial error
err=1;

iter=0;

while abs(err)>tol & iter<MAXIT

    switch kind
        case 1
            a=besselj(n,x0);
            b=besselj(n1,x0);
        case 2
            a=bessely(n,x0);
            b=bessely(n1,x0);
    end

    x02=x0*x0;

    err=2*a*x0*(n*a-b*x0)/(2*b*b*x02-a*b*x0*(4*n+1)+(n*n1+x02)*a*a);

    x=x0-err;
    x0=x;
    iter=iter+1;

end

if iter>MAXIT-1
    warning('Failed to converge to within tolerance. ',...
        'Try a different initial guess');
    x=inf;
end